/*****************************************************************************
**
** bpbdump.c - BIOS Paramater Block printing tool.  Based on format.c by
** Lou T. Garavaglia and Steven C. Cavender.
**
** Modified by: Rob Mitchelmore
**
******************************************************************************
*/

#include <mini.h>
#include <gemerror.h>

char usage[] = "usage: bpbdump [drive]\n";


#define GetBPB(a) bios(7,a)

#define BUFSIZ 10000
char buf[BUFSIZ];

#define BPB struct _bpb
BPB /* bios parameter block */
{
	int	recsiz;
	int	clsiz;
	int 	clsizb;
	int	rdlen;	/* root directory length in records */
	int	fsiz;	/* fat size in records */
	int	fatrec;	/* first fat record (of last fat) */
	int	datrec;	/* first data record */
	int	numcl;	/* number of data clusters available */
        int     b_flags;
} ;

/***************************************************************************/

main(p)
char* p;
{
	char  *d;
	int i, j, drv, rec, fs, f1, f2, nd;
	BPB *b;
	long err, sp;
	/* Get drive letter from command tail using base page pointer.*/
	
	if (p[0x80])
	{
 		p += 0x81;
		while (*p == ' ')	/* scan past initial blanks */
			p++;
		drv = (*p >= 'a') && (*p <= 'z') ? *p -'a' : *p - 'A';
		if (drv & 0xFFF0)	/* out of range? */
		{
			C_ConWS(usage);
			P_Term(1);
		}
	}
	else
	{
		C_ConWS(usage);
		P_Term(2);
	}	
	
	sp = S_State(0L);
	b = GetBPB(drv);
	printbpb(b);
	S_State(sp);

// 	buf[0] = FORMAT_DRIVE;
// 
// 	/* Format the disk.*/
// 
// 	while ( (err = F_IOCtl(5, drv+1, 1, &buf)) == E_CHNG )
// 		;
// 	if ( err )
// 		P_Term( (int)err );
// 
// 	/* Init the disk's FATS and root directory. */
// 
// 	for (i=0; i < BUFSIZ; i++)
// 		buf[i] = 0;
// 
// 	buf[0] = 0xF7;
// 	buf[1] = 0xFF;
// 	buf[2] = 0xFF;
// 
// 	sp = S_State(0L);
// 
// 	b = GetBPB(drv);
// 
// 	if (b->b_flags & 1)
// 		buf[3] = 0xFF;
// 
// 	f1 = b->fatrec - b->fsiz;
// 	f2 = b->fatrec;
// 	fs = b->fsiz;
// 	RWAbs(1,buf,fs,f1,drv);
// 	RWAbs(1,buf,fs,f2,drv);
// 	nd = b->recsiz / 32;
// 	d = buf;
// 
// 	for (i = 0; i < nd; i++)
// 	{
// 		*d++ = 0;
// 		for (j = 0; j < 31; j++)
// 			*d++ = 0;
// 	}
// 
// 	rec = f2 + fs;
// 
// 	for (i = 0; i < b->rdlen; i++, rec++)
// 		RWAbs(1,buf,1,rec,drv);
// 
// 	S_State(sp);
	
	P_Term(0);
}

printbpb(b)
BPB *b;
{
	C_ConWS("{ ");
	
	C_ConWI(b->recsiz);	
	C_ConWS(", ");
	C_ConWI(b->clsiz);
	C_ConWS(", ");
	C_ConWI(b->clsizb);
	C_ConWS(", ");
	C_ConWI(b->rdlen);
	C_ConWS(", ");
	C_ConWI(b->fsiz);
	C_ConWS(", ");
	C_ConWI(b->fatrec);
	C_ConWS(", ");
	C_ConWI(b->datrec);
	C_ConWS(", ");
	C_ConWI(b->numcl);
	C_ConWS(", ");
	C_ConWI(b->b_flags);
	
	C_ConWS(" }");
}

C_ConWI(i) 
int i;
{
	char itoabuf[15];
	itoa(i, itoabuf);
	C_ConWS(itoabuf);
}

itoa(n, str)
int n;
char* str;
{
	char buf[15];
	int i; // index into buffer
	int j; // index into output string 
	int negative;
	int rem;
	
	i = 0;
	negative = 0;
	
	
	// Terminating nul here.  We're constructing the string backwards so
	// put the nul first.
	buf[i++] = '\0';
	
	// Zero?
	if (n == 0) 
	{
		buf[i++] = '0';
	}
	
	// Negative?
	if (n < 0) 
	{
		negative = 1;
		n = -n;
	}
	
	// Iterate
	while (n > 0) {
		rem = n % 10;
		n -= rem;
		buf[i++] = rem + '0';
		n = n / 10;
	}
	
	if (negative) {
		buf[i++] = '-';
	}
	
	// Now read the string out backwards.
	j = 0;
	while (i >= 0) {
		str[j++] = buf[--i];
	}
}
